## implementation of Gibbs sampler for
## normal hierarchical model in
## Section 11.6 of BDA
##

## sample theta from model
## y ~ N(theta,sig2), theta ~ N(mu,tau2)
normal_mean_normal_prior <- function(y,sig2,mu,tau2){
    theta_mean <- (mu*sig2 + y*tau2)/(tau2 + sig2)
    theta_var <- (tau2*sig2)/(tau2 + sig2)
    return(rnorm(length(y),mean=theta_mean,sd=sqrt(theta_var)))
}

## sample theta from model
## y_i ~ N(theta,sig2) iid, p(theta) = 1
normal_mean_flat_prior <- function(y,sig2){
    return(rnorm(1,mean(y),sqrt(sig2/length(y))))
}

## sample from inverse chi-sq for variance
variance_sample <- function(a,b,nu){
    x <- rchisq(1,df=nu)
    return(sum((a-b)^2)/x)
}

## gibbs sampler with
##
##    arguments
##      y : responses
##  group : group[ii] is group of y[ii]
##      N : number of MCMC samples to draw
##
##      value
## params : samples from posterior
run_sampler <- function(y,group,N=1000){
    n <- length(y) ## total number of observations
    nj <- table(group) ## number obs / group
    J <- length(nj) ## number of groups
    params <- matrix(0,nrow=N,ncol=length(unique(group))+3)
    colnames(params) <- c("mu","tau2","sig2",paste0("theta",1:J))
    y_mean <- tapply(y,list(group),FUN=mean) ## means by group
    thetac <- y_mean ## starting group means at mean of obs in group
    muc <- mean(thetac) ## starting overall mean at mean of groups
    ## draw tau2, sig2, thetac, muc
    for(ii in 1:N){
        tau2c <- variance_sample(muc,thetac,J-1)
        sig2c <- variance_sample(thetac[group],y,n)
        thetac <- normal_mean_normal_prior(y_mean,sig2=sig2c/nj,mu=muc,tau2=tau2c)
        muc <- normal_mean_flat_prior(thetac,tau2c)
        params[ii,] <- c(muc,tau2c,sig2c,thetac)
    }
    return(params)
}
